@@ -7,6 +7,7 @@ from distutils.util import strtobool
 import json
 import numpy as np
 import uuid
+from enum import Enum
 from subprocess import check_call
 
 
@@ -27,8 +28,10 @@ def get_dot_body(treedict, parent=None, left=True):
             gstring += get_dot_body(treedict["left"], id)
             gstring += get_dot_body(treedict["right"], id, False)
     else:
-        val = str(int(treedict["value"])) if treedict["value"] - int(treedict["value"]) == 0 else str(round(treedict["value"], 3))
-        err = str(int(treedict["error"])) if treedict["error"] - int(treedict["error"]) == 0 else str(round(treedict["error"], 2))
+        val = str(int(treedict["value"])) if treedict["value"] - int(treedict["value"]) == 0 else str(
+            round(treedict["value"], 3))
+        err = str(int(treedict["error"])) if treedict["error"] - int(treedict["error"]) == 0 else str(
+            round(treedict["error"], 2))
         # maxi = max(len(val), len(err))
         # val = val if len(val) == maxi else val + (" " * (maxi - len(val)))
         # err = err if len(err) == maxi else err + (" " * (maxi - len(err)))
@@ -37,6 +40,18 @@ def get_dot_body(treedict, parent=None, left=True):
     return gstring
 
 
+class Cache_Type(Enum):
+    Cache_TrieItemset = 1
+    Cache_HashItemset = 2
+    Cache_HashCover = 3
+
+
+class Wipe_Type(Enum):
+    All = 1
+    Subnodes = 2
+    Reuses = 3
+
+
 class DL85Predictor(BaseEstimator):
     """ An optimal binary decision tree classifier.
 
@@ -46,6 +61,8 @@ class DL85Predictor(BaseEstimator):
         Maximum depth of the tree to be found
     min_sup : int, default=1
         Minimum number of examples per leaf
+    iterative : bool, default=False
+        Whether the search will be Iterative Deepening Search or not. By default, it is Depth First Search
     max_error : int, default=0
         Maximum allowed error. Default value stands for no bound. If no tree can be found that is strictly better, the model remains empty.
     stop_after_better : bool, default=False
@@ -60,6 +77,8 @@ class DL85Predictor(BaseEstimator):
         A parameter used to indicate heuristic function used to sort the items in ascending order
     repeat_sort : bool, default=False
         A parameter used to indicate whether the heuristic sort will be applied at each level of the lattice or only at the root
+    nps : bool, default=False
+        A parameter used to indicate if only optimal solutions should be stored in the cache.
     print_output : bool, default=False
         A parameter used to indicate if the search output will be printed or not
 
@@ -91,6 +110,8 @@ class DL85Predictor(BaseEstimator):
             min_sup=1,
             error_function=None,
             fast_error_function=None,
+            predict_error_function=None,
+            iterative=False,
             max_error=0,
             stop_after_better=False,
             time_limit=0,
@@ -99,13 +120,29 @@ class DL85Predictor(BaseEstimator):
             asc=False,
             repeat_sort=False,
             leaf_value_function=None,
+            nps=False,
             quiet=True,
-            print_output=False):
+            print_output=False,
+            cache_type=Cache_Type.Cache_TrieItemset,
+            maxcachesize=0,
+            wipe_type=Wipe_Type.Subnodes,
+            wipe_factor=0.5,
+            use_cache=True,
+            depth_two_special_algo=False,
+            use_ub=True,
+            similar_lb=False,
+            dynamic_branch=True,
+            similar_for_branching=False):
+
         self.max_depth = max_depth
         self.min_sup = min_sup
+        # self.max_estimators = max_estimators
         self.sample_weight = []
         self.error_function = error_function
         self.fast_error_function = fast_error_function
+        # self.example_weight_function = example_weight_function
+        # self.predict_error_function = predict_error_function
+        self.iterative = iterative
         self.max_error = max_error
         self.stop_after_better = stop_after_better
         self.time_limit = time_limit
@@ -114,10 +151,22 @@ class DL85Predictor(BaseEstimator):
         self.asc = asc
         self.repeat_sort = repeat_sort
         self.leaf_value_function = leaf_value_function
+        self.nps = nps
         self.quiet = quiet
         self.print_output = print_output
+        self.cache_type = cache_type
+        self.maxcachesize = maxcachesize
+        self.wipe_type = wipe_type
+        self.wipe_factor = wipe_factor
+        self.use_cache = use_cache
+        self.depth_two_special_algo = depth_two_special_algo
+        self.use_ub = use_ub
+        self.similar_lb = similar_lb
+        self.dynamic_branch = dynamic_branch
+        self.similar_for_branching = similar_for_branching
 
         self.tree_ = None
+        self.base_tree_ = None
         self.size_ = -1
         self.depth_ = -1
         self.error_ = -1
@@ -179,73 +228,70 @@ class DL85Predictor(BaseEstimator):
         # sys.path.insert(0, "../../")
         import dl85Optimizer
         # print(opt_func)
-        solution = dl85Optimizer.solve(data=X,
-                                       target=y,
-                                       tec_func_=opt_func,
-                                       sec_func_=opt_fast_func,
-                                       te_func_=opt_pred_func,
-                                       max_depth=self.max_depth,
-                                       min_sup=self.min_sup,
-                                       example_weights=self.sample_weight,
-                                       max_error=self.max_error,
-                                       stop_after_better=self.stop_after_better,
-                                       time_limit=self.time_limit,
-                                       verb=self.verbose,
-                                       desc=self.desc,
-                                       asc=self.asc,
-                                       repeat_sort=self.repeat_sort)
+        solution_str = dl85Optimizer.solve(data=X,
+                                           target=y,
+                                           tec_func_=opt_func,
+                                           sec_func_=opt_fast_func,
+                                           te_func_=opt_pred_func,
+                                           # exw_func_=self.example_weight_function,
+                                           # pred_func_=self.predict_error_function,
+                                           max_depth=self.max_depth,
+                                           min_sup=self.min_sup,
+                                           # max_estimators=self.max_estimators,
+                                           example_weights=self.sample_weight,
+                                           max_error=self.max_error,
+                                           stop_after_better=self.stop_after_better,
+                                           # iterative=self.iterative,
+                                           time_limit=self.time_limit,
+                                           verb=self.verbose,
+                                           desc=self.desc,
+                                           asc=self.asc,
+                                           repeat_sort=self.repeat_sort,
+                                           # bin_save=False,
+                                           # nps=self.nps,
+                                           predictor=predict,
+                                           cachetype=dl85Optimizer.CacheType.CacheTrieItemset if self.cache_type == Cache_Type.Cache_TrieItemset else dl85Optimizer.CacheType.CacheHashItemset if self.cache_type == Cache_Type.Cache_HashItemset else dl85Optimizer.CacheType.CacheHashCover,
+                                           cachesize=self.maxcachesize,
+                                           wipetype=dl85Optimizer.WipeType.All if self.wipe_type == Wipe_Type.All else dl85Optimizer.WipeType.Subnodes if self.wipe_type == Wipe_Type.Subnodes else dl85Optimizer.WipeType.Reuses,
+                                           wipefactor=self.wipe_factor,
+                                           withcache=self.use_cache,
+                                           usespecial=self.depth_two_special_algo,
+                                           useub=self.use_ub,
+                                           similar_lb=self.similar_lb,
+                                           dyn_branch=self.dynamic_branch,
+                                           similar_for_branching=self.similar_for_branching)
 
-        # if self.print_output:
-        #     print(solution)
+        solution = solution_str.rstrip("\n").splitlines()
 
-        solution = solution.rstrip("\n").splitlines()
-        self.sol_size = len(solution)
+        self.tree_ = json.loads(solution[-8].split('Tree: ')[1]) if "No such tree" not in solution[-8] else None
+        self.size_ = int(solution[-7].split(" ")[1])
+        self.depth_ = int(solution[-6].split(" ")[1])
+        self.error_ = float(solution[-5].split(" ")[1])
+        self.accuracy_ = float(solution[-4].split(" ")[1])
+        self.lattice_size_ = int(solution[-3].split(" ")[1])
+        self.runtime_ = float(solution[-2].split(" ")[1])
+        self.timeout_ = bool(strtobool(solution[-1].split(" ")[1]))
 
-        if self.sol_size == 10:  # solution found
-            self.base_tree_ = json.loads(solution[2].split('Tree: ')[1])
-            self.tree_ = json.loads(solution[2].split('Tree: ')[1])
-            self.size_ = int(solution[3].split(" ")[1])
-            self.depth_ = int(solution[4].split(" ")[1])
-            self.error_ = float(solution[5].split(" ")[1])
-            self.lattice_size_ = int(solution[7].split(" ")[1])
-            self.runtime_ = float(solution[8].split(" ")[1])
-            self.timeout_ = bool(strtobool(solution[9].split(" ")[1]))
-            if self.size_ >= 4 or self.max_error <= 0:
-                self.accuracy_ = float(solution[6].split(" ")[1])
+        if target_is_need:  # problem with target
+            self.classes_ = unique_labels(y)  # Store the classes seen during fit
 
-            # if sol_size == 8:  # without timeout
-            if self.size_ < 4 and self.max_error > 0:  # return just a leaf as fake solution
-                if not self.timeout_:
-                    print("DL8.5 fitting: Solution not found. However, a solution exists with error equal to the "
-                      "max error you specify as unreachable. Please increase your bound if you want to reach it.")
-                else:
-                    print("DL8.5 fitting: Timeout reached without solution. However, a solution exists with "
-                          "error equal to the max error you specify as unreachable. Please increase "
-                          "your bound if you want to reach it.")
-            else:
-                if not self.quiet:
-                    if not self.timeout_:
-                        print("DL8.5 fitting: Solution found")
-                    else:
-                        print("DL8.5 fitting: Timeout reached but solution found")
-
-            if target_is_need:  # problem with target
-                # Store the classes seen during fit
-                self.classes_ = unique_labels(y)
-
-        elif self.sol_size == 6:  # solution not found
-            self.lattice_size_ = int(solution[3].split(" ")[1])
-            self.runtime_ = float(solution[4].split(" ")[1])
-            self.timeout_ = bool(strtobool(solution[5].split(" ")[1]))
+        if self.tree_ is None:  # No solution
             if not self.timeout_:
-                print("DL8.5 fitting: Solution not found")
-            else:  # timeout reached
-                print("DL8.5 fitting: Timeout reached and solution not found")
+                print("DL8.5 fitting: Solution not found. However, a solution exists with error greater than or equal to the max error (exlusive upper bound) you specify. Please increase your bound if you want to find it.")
+            else:
+                print("DL8.5 fitting: Timeout reached without solution. Please increase the time limit and/or the max error (exlusive upper bound)")
+        else:
+            if not self.quiet:
+                if not self.timeout_:
+                    print("DL8.5 fitting: Solution found")
+                else:
+                    print("DL8.5 fitting: Timeout reached. The solution found may not be optimal")
 
         if hasattr(self, 'tree_') and self.tree_ is not None:
             # add transactions to nodes of the tree
             self.add_transactions_and_proba(X, y)
 
+            # label the leafs when a labelling function is provided
             if self.leaf_value_function is not None:
                 def search(node):
                     if self.is_leaf_node(node) is not True:
@@ -253,22 +299,16 @@ class DL85Predictor(BaseEstimator):
                         search(node['right'])
                     else:
                         node['value'] = self.leaf_value_function(node['transactions'])
+
                 node = self.tree_
                 search(node)
 
             self.remove_transactions()
 
+        self.base_tree_ = self.get_tree_without_transactions_and_probas() if self.tree_ is not None else None
+
         if self.print_output:
-            print(solution[0])
-            print(solution[1])
-            print("Tree:", self.tree_)
-            print("Size:", str(self.size_))
-            print("Depth:", str(self.depth_))
-            print("Error:", str(self.error_))
-            # print("Accuracy:", str(self.accuracy_))
-            print("LatticeSize:", str(self.lattice_size_))
-            print("Runtime:", str(self.runtime_))
-            print("Timeout:", str(self.timeout_))
+            print(solution_str)
 
         # Return the classifier
         self.is_fitted_ = True
@@ -312,7 +352,7 @@ class DL85Predictor(BaseEstimator):
         for i in range(X.shape[0]):
             pred.append(self.pred_value_on_dict(X[i, :]))
 
-        return np.array(pred)
+        return pred
 
     def pred_value_on_dict(self, instance, tree=None):
         node = tree if tree is not None else self.tree_
@@ -392,7 +432,8 @@ class DL85Predictor(BaseEstimator):
         names = [x[0] for x in node.items()]
         return 'error' in names
 
-    def add_transactions_and_proba(self, X, y=None):  # explore the decision tree found and add transactions to leaf nodes.
+    # explore the decision tree found and add transactions and class probabilities to leaf nodes.
+    def add_transactions_and_proba(self, X, y=None):
         def recurse(transactions, node, feature, positive):
             if transactions is None:
                 current_transactions = list(range(0, X.shape[0]))
@@ -403,7 +444,7 @@ class DL85Predictor(BaseEstimator):
                     node['proba'] = []
                     for c in self.classes_:
                         if c in count_dict:
-                            node['proba'].append(round(count_dict[c] / sum(counts), 2))
+                            node['proba'].append(count_dict[c] / sum(counts))
                         else:
                             node['proba'].append(0)
                 else:
@@ -425,7 +466,7 @@ class DL85Predictor(BaseEstimator):
                         node['proba'] = []
                         for c in self.classes_:
                             if c in count_dict:
-                                node['proba'].append(round(count_dict[c] / sum(counts), 2))
+                                node['proba'].append(count_dict[c] / sum(counts))
                             else:
                                 node['proba'].append(0)
                     else:
@@ -444,7 +485,7 @@ class DL85Predictor(BaseEstimator):
                         node['proba'] = []
                         for c in self.classes_:
                             if c in count_dict:
-                                node['proba'].append(round(count_dict[c] / sum(counts), 2))
+                                node['proba'].append(count_dict[c] / sum(counts))
                             else:
                                 node['proba'].append(0)
                     else:
@@ -456,7 +497,7 @@ class DL85Predictor(BaseEstimator):
         root_node = self.tree_
         recurse(None, root_node, None, None)
 
-    def tree_without_transactions(self):
+    def get_tree_without_transactions(self):
 
         def recurse(node):
             if 'transactions' in node and ('feat' in node.keys() or 'value' in node.keys()):
@@ -476,8 +517,48 @@ class DL85Predictor(BaseEstimator):
                 if 'left' in node.keys():
                     recurse(node['left'])
                     recurse(node['right'])
+
         recurse(self.tree_)
 
+    def get_tree_without_probas(self):
+
+        def recurse(node):
+            if 'proba' in node and ('feat' in node.keys() or 'value' in node.keys()):
+                del node['proba']
+                if 'left' in node.keys():
+                    recurse(node['left'])
+                    recurse(node['right'])
+
+        tree = dict(self.tree_)
+        recurse(tree)
+        return tree
+
+    def remove_probas(self):
+        def recurse(node):
+            if 'proba' in node and ('feat' in node.keys() or 'value' in node.keys()):
+                del node['proba']
+                if 'left' in node.keys():
+                    recurse(node['left'])
+                    recurse(node['right'])
+
+        recurse(self.tree_)
+
+    def get_tree_without_transactions_and_probas(self):
+
+        def recurse(node):
+            if 'feat' in node.keys() or 'value' in node.keys():
+                if 'proba' in node:
+                    del node['proba']
+                if 'transactions' in node:
+                    del node['transactions']
+                if 'left' in node.keys():
+                    recurse(node['left'])
+                    recurse(node['right'])
+
+        tree = dict(self.tree_)
+        recurse(tree)
+        return tree
+
     def export_graphviz(self):
         if self.is_fitted_ is False:  # fit method has not been called
             raise NotFittedError("Call fit method first" % {'name': type(self).__name__})
